#include "ActorSession.h"

namespace theater {

// stolen from ShareNetClient.h
enum
{
   NET_CLIENT_CONNECTED_TO_SERVER = 0,
   NET_CLIENT_DISCONNECTED_FROM_SERVER,
   NET_CLIENT_NEW_CHAT_TEXT,
   NET_CLIENT_CONNECT_BACK_REQUEST,
   NET_CLIENT_CHECK_FILE_COUNT,
   NET_CLIENT_PING,
   NET_CLIENT_PONG,
   NET_CLIENT_SCAN_THREAD_REPORT
};

ActorSession :: 
ActorSession(const String & name) : _name(name)
{
   // empty
}

ActorSession ::
~ActorSession()
{
   // empty
}

void 
ActorSession :: MessageReceivedFromGateway(MessageRef msgRef, void *)
{
   Message * msg = msgRef();
   if (msg)
   {
      switch(msg->what)
      {
         case NET_CLIENT_PING:  // respond to other clients' pings
         {
            String replyTo;
            if (msg->FindString("session", replyTo) == B_NO_ERROR)
            {
               msg->what = NET_CLIENT_PONG;

               String toString("/*/");
               toString += replyTo;
               toString += "/beshare";

               msg->RemoveName(PR_NAME_KEYS);
               msg->AddString(PR_NAME_KEYS, toString);

               msg->RemoveName("session");
               msg->AddString("session", "blah");  // server will set this correctly for us

               msg->RemoveName("version");
               msg->AddString("version", "MasterBesTheater Actor v" THEATER_VERSION_STRING);
               AddOutgoingMessage(msgRef);
            }
         }
         break;              

         default:
            // do nothing.  An actor's only response to the audience is pongs!
         break;
      }
   }
}

status_t
ActorSession :: AttachedToServer()
{
   if (StorageReflectSession::AttachedToServer() != B_NO_ERROR) return B_ERROR;
   
   MessageRef uploadMsg = GetMessageFromPool(PR_COMMAND_SETDATA);
   MessageRef nameMessage = GetMessageFromPool();
   if ((uploadMsg())&&(nameMessage()))
   {
      nameMessage()->AddString("name", _name);
      nameMessage()->AddBool("bot", true);
      nameMessage()->AddInt32("port", 0);  // BeShare requires this, even though we don't use it
      nameMessage()->AddString("version_name", "MasterBesTheater Actor");
      nameMessage()->AddString("version_num", THEATER_VERSION_STRING);
      uploadMsg()->AddMessage("beshare/name", nameMessage);
      return AddOutgoingMessage(uploadMsg);
   }
   else return B_ERROR;
}

status_t
ActorSession :: ReciteLine(const String & line)
{
   MessageRef chatMessage = GetMessageFromPool(NET_CLIENT_NEW_CHAT_TEXT);
   if (chatMessage())
   {
      chatMessage()->AddString(PR_NAME_KEYS, "beshare");  // sent only to beshare compatible clients
      chatMessage()->AddString("session", "blah");        // will be set by server
      chatMessage()->AddString("text", line);
      return AddOutgoingMessage(chatMessage);
   }
   else return B_ERROR;
}

status_t
ActorSession :: SetStatus(const String & newStatus)
{
   MessageRef uploadMsg = GetMessageFromPool(PR_COMMAND_SETDATA);
   MessageRef statMessage = GetMessageFromPool();
   if ((uploadMsg())&&(statMessage()))
   {
      statMessage()->AddString("userstatus", newStatus);
      uploadMsg()->AddMessage("beshare/userstatus", statMessage);
      return AddOutgoingMessage(uploadMsg);
   }
   else return B_ERROR;
}

};  // end namespace theater
