#ifndef DirectorSession_h
#define DirectorSession_h

#include "ActorSession.h"
#include "TheaterConstants.h"
#include "iogateway/PlainTextMessageIOGateway.h"

namespace theater {

/** This session reads the script and tells the actors what to do. */
class DirectorSession : public ActorSession
{
public:
   /** prefs contains the settings from the theater.settings file 
     * @param instant If true, the lines will be read as fast as possible.  Otherwise they will be read at human speed.
     * @param scriptIO The script's text I/O stream to read from.
     * @param server The MUSCLE server to log into and host the play on.
     * @param port The port to log into.
     */
   DirectorSession(bool instant, DataIORef scriptIO, const char * server, uint16 port);

   /** Destructor */
   virtual ~DirectorSession();

   /** Overridden to add ourself to the actor's table */
   virtual status_t AttachedToServer();

   /** Overridden to kill the bot if we are killed */
   virtual void AboutToDetachFromServer();

   /** Implemented to wait unti the next line of the script should be parsed */
   virtual void Pulse(uint64 now, uint64 schedTime);

   /** Overridden to read and execute the next line of the script */
   virtual uint64 GetPulseTime(uint64 now, uint64 prevResult);

   /** When in stdin-reading mode, the StdinSession can call this to give us more text */
   void MessageReceivedFromStdin(MessageRef msg);

   /** When in stdin-reading mode, the StdinSession can call this to tell us he's going away */
   void StdinClosed();

protected:
   virtual const char * GetTypeName() const {return "Director";}

private:
   // Should return the number of milliseconds the line reading will take, or -1 on failure.
   int32 ExecuteScriptLine(const String & line, bool scanForNames);
   int32 ExecuteBufferedLine();
   int32 SetCurrentSession(const String & newSessionID, const String & line, int32 startBytes);

   String _server;
   uint16 _port;

   PlainTextMessageIOGateway _scriptGateway;
   QueueGatewayMessageReceiver _scriptQueue;
   MessageRef _currentScriptMessage;

   Hashtable<String, String> _actorNameToSessionID;
   String _currentSessionID;
 
   String _bufferedLine;

   bool _instant;
   uint64 _nextPulseTime;

   uint32 _lineNumber;
   uint32 _pace;

   bool _scriptEnded;
};

};  // end namespace theater

#endif
